# Generates a 64-character secret for the tunnel.
# Using `random_password` means the result is treated as sensitive and, thus,
# not displayed in console output. Refer to: https://registry.terraform.io/providers/hashicorp/random/latest/docs/resources/password
resource "random_password" "tunnel_secret" {
  length = 64
}

# Creates a new locally-managed tunnel for the VM.
resource "cloudflare_zero_trust_tunnel_cloudflared" "auto_tunnel" {
  account_id = var.cloudflare_account_id
  name       = "${var.lxd_container_name}@${var.lxd_machine_name}"
  secret     = base64sha256(random_password.tunnel_secret.result)
}

# Creates the CNAME record that routes ${var.subdomain}-ssh.${var.cloudflare_zone} to the tunnel.
resource "cloudflare_record" "ssh_app" {
  zone_id = var.cloudflare_zone_id
  name    = var.subdomain == "" ? "ssh" : "${var.subdomain}-ssh"
  content   = "${cloudflare_zero_trust_tunnel_cloudflared.auto_tunnel.id}.cfargotunnel.com"
  type    = "CNAME"
  proxied = true
}

resource "cloudflare_record" "web_app" {
  zone_id = var.cloudflare_zone_id
  name    = var.subdomain == "" ? "@" : var.subdomain
  content   = "${cloudflare_zero_trust_tunnel_cloudflared.auto_tunnel.id}.cfargotunnel.com"
  type    = "CNAME"
  proxied = true
}
